Package: thredds
Title: Crawler for Navigating THREDDS Catalogs
Description: Provides a crawler for programmatically navigating THREDDS Data Server (<https://www.unidata.ucar.edu/software/tds/>) 
  catalogs, and access dataset metadata and resources.
Version: 0.1-4
Date: 2023-09-01
Depends: R (>= 3.0)
Imports: R6, rlang, magrittr, httr, xml2
Suggests: ncdf4, testthat
Maintainer: Emmanuel Blondel <emmanuel.blondel1@gmail.com>
Authors@R: c(
  person("Ben", "Tupper", role = c("aut"), email = "btupper@bigelow.org"),
  person("Emmanuel", "Blondel", role = c("aut", "cre"), email = "emmanuel.blondel1@gmail.com", comment = c(ORCID = "0000-0002-5870-5762")),
  person("Bigelow Laboratory for Ocean Sciences", role = c("cph"), email = "info@bigelow.org"))
URL: https://github.com/BigelowLab/thredds,
        https://www.unidata.ucar.edu/software/tds/
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.2.3
NeedsCompilation: no
Packaged: 2023-09-01 09:01:43 UTC; emman
Author: Ben Tupper [aut],
  Emmanuel Blondel [aut, cre] (<https://orcid.org/0000-0002-5870-5762>),
  Bigelow Laboratory for Ocean Sciences [cph]
Repository: CRAN
Date/Publication: 2023-09-01 09:40:01 UTC
Built: R 4.6.0; ; 2026-04-26 10:48:50 UTC; unix
