# Generated by roxygen2: do not edit by hand

export(CatalogNode)
export(DatasetNode)
export(ServiceNode)
export(ThreddsNode)
export(build_xpath)
export(get_catalog)
export(get_xml_ns)
export(grepl_it)
export(is_xmlNode)
export(parse_node)
export(xmlString)
export(xml_children_names)
export(xml_id)
import(R6)
import(httr)
import(rlang)
importFrom(magrittr,"%>%")
